from fixieai.client.client import FixieClient
from fixieai.client.client import get_agents
from fixieai.client.client import get_client
from fixieai.client.client import get_embeds
from fixieai.client.client import query
from fixieai.client.console import Console

__all__ = [
    "FixieClient",
    "get_agents",
    "get_client",
    "get_embeds",
    "query",
    "Console",
]
